;
;	IKBD 6301 interrupt routine
;	(c) 2010 by Simon Sunnyboy / Paradize <marndt@asmsoftware.de>
;	http://paradize.atari.org/
;
;	derived from a similar routine Copyright (C) 2002	Patrice Mandin
;
;
;	This library is free software; you can redistribute it and/or
;	modify it under the terms of the GNU Lesser General Public
;	License as published by the Free Software Foundation; either
;	version 2.1 of the License, or (at your option) any later version.
;
;	This library is distributed in the hope that it will be useful,
;	but WITHOUT ANY WARRANTY; without even the implied warranty of
;	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;	Lesser General Public License for more details.
;
;	You should have received a copy of the GNU Lesser General Public
;	License along with this library; if not, write to the Free Software
;	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
;

export	IKBD_Install
export	IKBD_Uninstall
export	IKBD_Keyboard
export	IKBD_MouseX
export	IKBD_MouseY
export	IKBD_MouseB
export	IKBD_Joystick0
export	IKBD_Joystick1
export	IKBD_MouseOff
export	IKBD_MouseOn

	TEXT
	dc.b "Original code (c) 2002 by Patrice Mandin"
	EVEN
; ** installs the IKBD handler
IKBD_Install:
	movem.l	d0-d1/a0-a1,-(sp)

	; Disable all interrupts

	move.w	#0x2700,sr

	; Save MFP registers used for ACIA interrupt

	lea	$fffffa00.w,a0
	btst	#6,$9(a0)
	sne		ikbd_ierb
	btst	#6,$15(a0)
	sne		ikbd_imrb

	; Set our MFP routine

	move.l	$118.w,old_ikbd
	move.l	#ikbd,$118.w
	bset	#6,$fffffa09.w	; IERB
	bset	#6,$fffffa15.w	; IMRB

	; Set relative mouse motion mode
	; needed because running a .tos or .ttp program
	; disable the mouse (entering VT52 textmode)

	bsr IKBD_MouseOn

	; Re-enable interrupts

	move.w	#$2300,sr

	movem.l	(sp)+,d0-d1/a0-a1
	rts

; ** reinstalls the old IKBD vector
IKBD_Uninstall:
	move.l	a0,-(sp)

	; Disable interrupts

	move.w	#0x2700,sr

	; Restore previous MFP registers

	lea	$fffffa00.w,a0

	bclr	#6,$9(a0)
	tst.b	ikbd_ierb
	beq.s	ikbd_restoreierb
	bset	#6,$9(a0)
ikbd_restoreierb:

	bclr	#6,$15(a0)
	tst.b	ikbd_imrb
	beq.s	ikbd_restoreimrb
	bset	#6,$15(a0)
ikbd_restoreimrb:

	move.l	old_ikbd,$118.w

	; Clear keyboard buffer

	lea	$fffffc00.w,a0
ikbd_clearbuffer:
	btst	#0,(a0)
	beq.s	ikbd_buffercleared
	tst.b	2(a0)
	bra.s	ikbd_clearbuffer
ikbd_buffercleared:

	; Re-enable interrupts

	move.w	#$2300,sr

	move.l	(sp)+,a0
	rts
; *** mouse off routine -> set joystick reporting ***
IKBD_MouseOff:
	move.b #$14,$fffffc02.w
	rts
; *** mouse on routine -> disable joystick and report mouse again ***
IKBD_MouseOn:
	move.b #$08,$fffffc02.w
	rts
; custom IKBD vector
	EVEN
	dc.b	"XBRA"
	dc.b	"IKBD"
old_ikbd:	ds.l	1

ikbd:
	; test if byte coming from IKBD or MIDI

	btst	#0,$fffffc00.w
	beq.s	ikbd_endit

	movem.l	d0-d1/a0,-(sp)

	move.b	$fffffc02.w,d0

	; Joystick packet ?
	
	cmp.b	#$ff,d0
	beq.s	ikbd_yes_joystick1

	; Mouse packet ?

	cmp.b	#$f8,d0
	bmi.s	ikbd_no_mouse
	cmp.b	#$fc,d0
	bpl.s	ikbd_no_mouse

ikbd_yes_mouse:
	and.b	#3,d0
	move.b	d0,IKBD_MouseB

	move.l	#ikbd_mousex,$118.w    ; queue different irq vector for mouse packet
	bra.s	ikbd_endit_stack

ikbd_yes_joystick1:
	move.l	#ikbd_poll_joystick1,$118.w  ; queue joystick irq for next packet
	bra.s	ikbd_endit_stack

	; Keyboard press/release

ikbd_no_mouse:
	move.b	d0,d1
	lea	IKBD_Keyboard,a0
	and.l	#$7f,d1			; mask scancode
	tas	d0			
	spl	0(a0,d1.w)		; store -1 in scancode table

ikbd_endit_stack:
	movem.l	(sp)+,d0-d1/a0
ikbd_endit:
	bclr	#6,$fffffa11.w		; acknowledge IKBD interrupt
	rte				; ... and terminate the ISR

ikbd_mousex:
	; mouse packet?

	btst	#0,$fffffc00.w
	beq.s	ikbd_endit

	move.w	d0,-(sp)

	move.b	$fffffc02.w,d0
	ext.w	d0
	add.w	d0,IKBD_MouseX

	move.w	(sp)+,d0

	move.l	#ikbd_mousey,$118.w
	bra.s	ikbd_endit

ikbd_mousey:
	; mouse packet?

	btst	#0,$fffffc00.w
	beq.s	ikbd_endit

	move.w	d0,-(sp)

	move.b	$fffffc02.w,d0
	ext.w	d0
	add.w	d0,IKBD_MouseY

	move.w	(sp)+,d0

	move.l	#ikbd,$118.w
	bra.s	ikbd_endit

ikbd_poll_joystick1:
	; test if byte coming from joystick

	btst	#0,$fffffc00.w
	beq.s	ikbd_endit

	move.b	$fffffc02.w,IKBD_Joystick1

	move.l	#ikbd,$118.w
	bra.s	ikbd_endit

	BSS
	EVEN
	
IKBD_Keyboard:	ds.b 128
	EVEN
IKBD_MouseX:	ds.w 1
IKBD_MouseY:	ds.w 1
	EVEN
IKBD_MouseB:	ds.b 1
IKBD_Joystick0:	ds.b 1
IKBD_Joystick1: ds.b 1
	EVEN
ikbd_ierb:	ds.b 1
ikbd_imrb:	ds.b 1
	END
	