/*
 *
 *	IKBD 6301 interrupt routine
 *	(c) 2010 by Simon Sunnyboy / Paradize <marndt@asmsoftware.de>
 *	http://paradize.atari.org/
 *
 *	handling of key presses
 *
 *	derived from a similar routine Copyright (C) 2002	Patrice Mandin
 *
 *	This library is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU Lesser General Public
 *	License as published by the Free Software Foundation; either
 *	version 2.1 of the License, or (at your option) any later version.
 *
 *	This library is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *	Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with this library; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */

#include "src\ikbd.h"

#define TIMER (*(unsigned long*)0x000004baUL)

/* flushes all pending keyboard and joystick events */
void IKBD_Flush()
{
	unsigned char i;
	unsigned char flush_ok;
	unsigned long tick;
	
	do
	{
		tick = TIMER;		
		flush_ok = (  (IKBD_MouseB    == 0)
		            &&(IKBD_Joystick0 == 0)
		            &&(IKBD_Joystick1 == 0));
		            
		for(i=0; i<128; i++)
		{
			if(IKBD_Keyboard[i] != IKBD_KEY_RELEASED)
			{
				flush_ok = 0;
				break;			/* allow some time to pass */
			}
		}
		/* await Timer C to count */
		while((tick + 10) < TIMER )
		{
			/* wait for time */
		}
		
	} while(!flush_ok);
	return;
}

/* returns and debounces a key by scancode */
unsigned char IKBD_IsKeyPressed(unsigned char scancode)
{
	unsigned char keystatus;

	if(scancode & 0x80)
	{
		/* upper bit of scancode set -> scancode is invalid */
		return(IKBD_KEY_RELEASED);
	}
	else
	{
		keystatus = IKBD_Keyboard[scancode];
		
		if((keystatus == IKBD_KEY_PRESSED)||(keystatus == IKBD_KEY_RELEASED))
		{
			/* current key in defined state -> read it and set as read */
			IKBD_Keyboard[scancode] == IKBD_KEY_UNDEFINED;
			return(keystatus);
		}
	}	
	/* undefined key state -> means key not pressed */
	return(IKBD_KEY_RELEASED);
}
