/*
 *
 *	IKBD 6301 interrupt routine
 *	(c) 2010 by Simon Sunnyboy / Paradize <marndt@asmsoftware.de>
 *	http://paradize.atari.org/
 *
 *	mouse data collection
 *
 *	derived from a similar routine Copyright (C) 2002	Patrice Mandin
 *
 *	This library is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU Lesser General Public
 *	License as published by the Free Software Foundation; either
 *	version 2.1 of the License, or (at your option) any later version.
 *
 *	This library is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *	Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with this library; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */

#include "src/ikbd.h"

/* variables */
IKBD_MouseData	IKBD_Mouse;          /* mouse data */

unsigned char MouseThreshold = 128;  /* mouse threshold (255 = 200%) */

/* reads the accumulated mouse packets and updates the mouse position */
void IKBD_ReadMouse()
{
	signed short xoffset, yoffset;
	
	if(MouseThreshold > 0)
	{
		/* scale the mouse movement by the configurable threshold */
		xoffset = (signed short)((signed long)(IKBD_MouseX * MouseThreshold) / 128);
		yoffset = (signed short)((signed long)(IKBD_MouseY * MouseThreshold) / 128);
	}
	else
	{
		/* 0% scaling is not allowed -> scale as 100% */
		xoffset = IKBD_MouseX;
		yoffset = IKBD_MouseY;
	}

	IKBD_Mouse.b = IKBD_MouseB;
	IKBD_Mouse.x = IKBD_Mouse.x + xoffset;
	IKBD_Mouse.y = IKBD_Mouse.y + yoffset;
	
	if(IKBD_Mouse.limit != 0)
	{
		/* mouse coordinates limitation is active? */
		if(IKBD_Mouse.x < 0)
		{
			IKBD_Mouse.x = 0;
		}
		else if(IKBD_Mouse.x > IKBD_Mouse.w)
		{
			IKBD_Mouse.x = IKBD_Mouse.w;
		}
		if(IKBD_Mouse.y < 0)
		{
			IKBD_Mouse.y = 0;
		}
		else if(IKBD_Mouse.y > IKBD_Mouse.h)
		{
			IKBD_Mouse.y = IKBD_Mouse.h;
		}
	}
	
	/* acknowledge accumulated mouse vector */
	IKBD_MouseX = 0;
	IKBD_MouseY = 0;
	return;
}

/* allows to configure teh scaling of relative mouse movement */
void IKBD_SetMouseThreshold(unsigned char new_threshold)
{
	MouseThreshold = new_threshold;
	return;
}

/* allows to set current mouse position and define the screen size */
void IKBD_SetMouseOrigin(signed short x, signed short y, unsigned short w, unsigned short h)
{
	IKBD_Mouse.x = x;
	IKBD_Mouse.y = y;
	IKBD_Mouse.w = w;
	IKBD_Mouse.h = h;
	IKBD_Mouse.b = 0;
	IKBD_Mouse.limit = 1;
	IKBD_MouseX = 0;
	IKBD_MouseY = 0;
	return;
}
